<?php

namespace Dropp\Woocommerce;

use GuzzleHttp\Client;

class ApiClient {

	/**
	 * @var Client|bool the configured HTTP client
	 */
	private static $client;

	/**
	 * Get a client for Dropp API ready to use.
	 *
	 * @return Client
	 */
	public static function get_client( ShippingMethod $shipping_method = null ) {
		if ( ! self::$client ) {
			self::$client = self::create_client( $shipping_method );
		}

		return self::$client;
	}

	/**
	 * Creates a new client with options from shipping method, if provided, otherwise
	 * it'll lookup for the proper shipping method instance.
	 *
	 * @param ShippingMethod|null $shipping_method
	 *
	 * @return Client|null
	 */
	private static function create_client( ShippingMethod $shipping_method = null ) {
		// if shipping method is provided, we don't need to search for it
		if ( $shipping_method === null ) {
			$shipping_method = self::find_dropp_shipping_method();

			if ( $shipping_method === null ) {
				return null;
			}
		}

		return new Client( [
			'base_uri' => sprintf( '%s/shops/%d/', $shipping_method->api_url, $shipping_method->get_option( 'shop_id' ) ),
			'headers'  => [
				'X-Api-Key'    => $shipping_method->get_option( 'api_key' ),
				'Content-type' => 'Application/json',
				'Accept'       => 'application/json',
			],
		] );
	}

	/**
	 * Lookup for the Dropp shipping method instance and its configurations.
	 *
	 * @return ShippingMethod|null
	 */
	private static function find_dropp_shipping_method() {
		$methods = WC()->shipping->get_shipping_methods();

		foreach ( $methods as $method ) {
			if ( $method instanceof ShippingMethod ) {
				return $method;
			}
		}

		return null;
	}
}