<?php

namespace Dropp\Woocommerce;

use GuzzleHttp\Exception\GuzzleException;
use WC_Order;
use WC_Product;

class Payment {

	/**
	 * Register the hook for when order changes its state and register the data to be sent
	 * to Dropp system.
	 */
	public function register_hooks() {
		add_action( 'woocommerce_order_status_' . $this->get_order_state_to_listen(), [ $this, 'push_order' ] );
		add_action( 'woocommerce_payment_complete', [ $this, 'push_order' ] );
		add_action( 'woocommerce_new_order', [ $this, 'set_timeslot_metadata' ] );
	}

	/**
	 * Return the order status to listen to.
	 *
	 * @return string
	 */
	private function get_order_state_to_listen() {
		$shipping_method = new ShippingMethod();

		return str_replace( 'wc-', '', $shipping_method->push_order_on_status );
	}

	/**
	 * Returns if the shipping method id for a given order is Dropp.
	 *
	 * @param WC_Order $order
	 *
	 * @return bool
	 */
	private function is_dropp_order( WC_Order $order ) {
		$items           = $order->get_items( 'shipping' );
		$shipping_method = reset( $items )->get_method_id();

		return $shipping_method === 'with_dropp';
	}

	/**
	 * Return the prepared data for the selected timeslot in the current order.
	 *
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	private function get_selected_timeslot_data( WC_Order $order ) {
		$timeslot = json_decode( $order->get_meta( 'dropp_selected_timeslot' ), true );

		if ( ! $timeslot ) {
			return [];
		}

		return [
			'phone'                 => $timeslot['deliveryContactPhone'] ?: ( $order->get_shipping_phone() ? $order->get_shipping_phone() : $order->get_billing_phone() ),
			'deliveryTimeFromUtc'   => $timeslot['deliveryTimeFromUtc'],
			'deliveryTimeToUtc'     => $timeslot['deliveryTimeToUtc'],
			'deliveryContactMethod' => $timeslot['deliveryContactMethod'],
			'deliveryContactNote'   => $timeslot['deliveryContactNote'],
		];
	}

	/**
	 * Builds the data payload for a given order.
	 *
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	private function build_push_order_payload( WC_Order $order ) {
		return array_merge(
			[
				'email'       => $order->get_billing_email(),
				'phone'       => $order->get_shipping_phone(),
				'firstName'   => $order->get_shipping_first_name(),
				'lastName'    => $order->get_shipping_last_name(),
				'street1'     => $order->get_shipping_address_1(),
				'street2'     => $order->get_shipping_address_1(),
				'city'        => $order->get_shipping_city(),
				'postalCode'  => $order->get_shipping_postcode(),
				'iso2Country' => $order->get_shipping_country(),
				'amount'      => $order->get_total(),
				'currency'    => $order->get_currency(),
				'description' => $order->get_customer_note(),
				'externalId'  => $order->get_id(),
				'items'       => array_map(
					function ( $item ) {
						$product_variation_id = $item['variation_id'];

						// Check if product has variation.
						if ( $product_variation_id ) {
							$product = new WC_Product( $item['variation_id'] );
						} else {
							$product = new WC_Product( $item['product_id'] );
						}

						return [
							'quantity' => $item->get_quantity(),
							'sku'      => $product->get_sku(),
							'name'     => $item->get_name(),
							'title'    => $item->get_name(),
						];
					},
					$order->get_items()
				),
			],
			$this->get_selected_timeslot_data( $order )
		);
	}

	/**
	 * Read the metadata if the order has been pushed for a given order.
	 *
	 * @param WC_Order $order
	 *
	 * @return bool
	 */
	private function has_order_been_pushed( WC_Order $order ) {
		return $order->get_meta( 'dropp_order_pushed' );
	}

	/**
	 * Save the metadata of the order as pushed to Dropp.
	 *
	 * @param WC_Order $order
	 */
	private function set_order_as_pushed( WC_Order $order ) {
		$order->update_meta_data( 'dropp_order_pushed', 'yes' );
		$order->save();
	}

	/**
	 * Add a note to the order so it's visible to the store admin.
	 *
	 * @param WC_Order $order
	 */
	private function add_note_to_order( WC_Order $order, $msg ) {
		$order->add_order_note( $msg );
	}

	/**
	 * Pushes the current order to Dropp.
	 *
	 * @param int $order_id
	 *
	 * @return void
	 */
	public function push_order( $order_id ) {
		$order = wc_get_order( $order_id );

		if ( ! $this->is_dropp_order( $order ) || $this->has_order_been_pushed( $order ) ) {
			return;
		}

		$client = ApiClient::get_client();
		$data   = $this->build_push_order_payload( $order );

		try {
			$response = $client->post( 'orders', [ 'json' => $data ] );

			if ( $response->getStatusCode() !== 200 ) {
				// TODO: handle status not 200
				return;
			}

			$this->set_order_as_pushed( $order );
			$this->add_note_to_order( $order, __( 'Order pushed to Dropp', 'woocommerce-with-dropp' ) );
		} catch ( GuzzleException $e ) {
			$this->add_note_to_order( $order, printf( __( 'Error pushing order to Dropp: %s', 'woocommerce-with-dropp' ), $e->getMessage() ) );
		}
	}

	/**
	 * Reads the selected timeslot from the session and add as metadata of the order.
	 *
	 * @param $order_id
	 *
	 * @return void
	 */
	public function set_timeslot_metadata( $order_id ) {
		$order = wc_get_order( $order_id );

		$session  = new Session();
		$timeslot = $session->get_selected_timeslot();

		// if timeslot is not in the session, ignore
		if ( ! $timeslot ) {
			return;
		}

		$order->update_meta_data( 'dropp_selected_timeslot', json_encode( $timeslot ) );
		$order->save();
	}
}